@@ -54,6 +54,9 @@ extern "C" {
 #include <algorithm>
 #include <string>
 #include <random>
+#include <iomanip>
+#include <cassert>
+#include <memory>
 
 #include "options.h"
 #include "vomsxml.h"
@@ -459,7 +462,7 @@ Client::Client(int argc, char ** argv) :
     exit(1);
   }
 
-  Print(DEBUG) << "Number of bits in key :" << bits << std::endl; 
+  Print(DEBUG) << "Number of bits in key: " << bits << std::endl; 
   
   /* parse valid options */
 
@@ -671,8 +674,6 @@ int Client::Run()
 
       /* create a temporary proxy to contact the server */  
       if (!noregen) {
-        Print(INFO) << "Creating temporary proxy " << std::flush;
-        Print(DEBUG) << "to " << proxyfile << " " << std::flush;
 
         int tmp = hours;
         hours = 1;
@@ -682,8 +683,8 @@ int Client::Run()
       }
       
       /* contact server */
-      Print(INFO) << "Contacting " << " " << beg->host << ":" << beg->port
-                  << " [" << beg->contact << "] \"" << beg->vo << "\"" << std::flush;
+      Print(INFO) << "Contacting " << beg->host << ":" << beg->port
+                  << " [" << beg->contact << "] \"" << beg->vo << "\"..." << std::flush;
 
       int status = v->ContactRaw(beg->host, beg->port, beg->contact, command, buffer, version, timeout);
 
@@ -731,7 +732,7 @@ int Client::Run()
         Print(INFO) << std::endl << "Trying next server for " << beg->nick << "." << std::endl;
       }
       else {
-        Print(ERROR) << std::endl << "None of the contacted servers for " << beg->vo << " were capable\nof returning a valid AC for the user." << std::endl;
+        Print(ERROR) << "\nNone of the contacted servers for " << beg->vo << " were capable of returning a valid AC for the user.\n";
         if (!noregen) 
           unlink(proxyfile.c_str());
         return 1;
@@ -778,19 +779,12 @@ int Client::Run()
   }
   
   /* create a proxy containing the data retrieved from VOMS servers */
-  
-  Print(INFO)  << "Creating proxy " << std::flush; 
-  Print(DEBUG) << "to " << proxyfile << " " << std::flush;
 
   if (CreateProxy(data, aclist, proxyver)) {
     goto err2;
   }
-  //  else  {
-    //    free(aclist);
-    //    aclist = NULL;
-  //  }
   
-  Print(INFO) << "\n" << std::flush;
+  Print(INFO) << '\n';
 
   /* unset environment */
   
@@ -799,12 +793,7 @@ int Client::Run()
   else {
     setenv("X509_USER_PROXY", oldenv, 1);
   }
-  
-  /* assure user certificate is not expired or going to, else advise but still create proxy */
-  
-  if (Test())
-    return 1;
-  
+ 
   return Verify();
 
  err:
@@ -817,10 +806,46 @@ int Client::Run()
   return 1;
 }
 
+namespace {
+
+// generate a string preferably in local time, with TZ indication
+std::string to_string(const ASN1_TIME *time)
+{
+  assert(time != nullptr);
+
+  tm tm_utc;
+  ASN1_TIME_to_tm(time, &tm_utc);
+  std::ostringstream os;
+#ifdef HAVE_TIMEGM
+  time_t t_utc = timegm(&tm_utc);
+  tm *tm_ptr = localtime(&t_utc);
+  os << std::put_time(tm_ptr, "%c %Z");
+#else
+  os << std::put_time(&tm_utc, "%c GMT");
+#endif
+  return os.str();
+}
+
+}
+
 bool Client::CreateProxy(std::string data, AC ** aclist, int version) 
 {
-  struct VOMSProxyArguments *args = VOMS_MakeProxyArguments();
-  int ret = 0;
+  using ArgsPtr = std::unique_ptr<VOMSProxyArguments, void (*)(VOMSProxyArguments *)>;
+  ArgsPtr args{
+      VOMS_MakeProxyArguments(),
+      [](VOMSProxyArguments* args)
+      {
+        free(args->proxyfilename);
+        free(args->policyfile);
+        free(args->policylang);
+        free(args->voID);
+        free(args->filename);
+
+        VOMS_FreeProxyArguments(args);
+      }
+  };
+
+  int ret = -1;
 
   if (args) {
     args->proxyfilename = strdup(proxyfile.c_str());
@@ -848,13 +873,20 @@ bool Client::CreateProxy(std::string data, AC ** aclist, int version)
     args->limited       = limit_proxy;
 
     args->voID          = strdup(voID.c_str());
-    args->callback      = kpcallback;
     int warn = 0;
     void *additional = NULL;
 
-    struct VOMSProxy *proxy = VOMS_MakeProxy(args, &warn, &additional);
+    // we are creating a temporary proxy if there is no AC or
+    // if this is a plain Grid proxy (i.e. there was no -voms)
+    bool const temporary = aclist == nullptr && !vomses.empty();
+    Print(DEBUG) << "Creating" << (temporary ? " temporary " : " ")
+                 << "proxy in " << proxyfile << "... " << std::flush;
 
-    PrintProxyCreationError(warn, additional);
+    using ProxyPtr = std::unique_ptr<VOMSProxy, void(*)(VOMSProxy*)>;
+    ProxyPtr proxy{
+      VOMS_MakeProxy(args.get(), &warn, &additional),
+      [](VOMSProxy* p) { VOMS_FreeProxy(p); }
+    };
 
     if (proxy) {
       /* In case of success, OpenSSL routines have already automagically 
@@ -863,23 +895,29 @@ bool Client::CreateProxy(std::string data, AC ** aclist, int version)
         free(args->aclist);
         this->aclist = NULL;
       }
-      ret = VOMS_WriteProxy(proxyfile.c_str(), proxy);
-      if (ret == -1) 
-        Print(ERROR) << "\nERROR: Cannot write proxy to: " << proxyfile << std::endl << std::flush;
+      ret = VOMS_WriteProxy(proxyfile.c_str(), proxy.get());
+      if (ret == -1) {
+        Print(DEBUG) << "Failed\n";
+        PrintProxyCreationError(warn, additional);
+        if (!temporary)
+        {
+          Print(ERROR) << "\nERROR: Cannot write proxy to: " << proxyfile << '\n';
+        }
+      } else {
+        Print(DEBUG) << "Done\n";
+        PrintProxyCreationError(warn, additional);
+
+        if (!temporary)
+        {
+          Print(INFO) << "\nCreated proxy in " << proxyfile
+                      << ".\n\nYour proxy is valid until "
+                      << to_string(X509_get0_notAfter(proxy->cert)) << '\n';
+        }
+      }
+    } else {
+      Print(DEBUG) << "Failed\n";
+      Print(ERROR) << OpenSSLError(true) << '\n';
     }
-    
-
-    if (ret != -1)
-      Print(INFO) << " Done" << std::endl << std::flush;
-
-    VOMS_FreeProxy(proxy);
-    free(args->proxyfilename);
-    free(args->policyfile);
-    free(args->policylang);
-    free(args->voID);
-    free(args->filename);
-
-    VOMS_FreeProxyArguments(args);
   }
 
   return ret == -1;
@@ -891,9 +929,9 @@ void Client::PrintProxyCreationError(int error, void *additional)
 
   if (msg) {
     if (PROXY_ERROR_IS_WARNING(error))
-      Print(DEBUG) << msg;
+      Print(DEBUG) << msg << '\n';
     else
-      Print(ERROR) << msg;
+      Print(ERROR) << msg << '\n';
     free(msg);
   }
 }
@@ -1018,36 +1056,6 @@ int Client::Verify()
 
 bool Client::Test() 
 {
-  ASN1_UTCTIME * asn1_time = ASN1_UTCTIME_new();
-  X509_gmtime_adj(asn1_time, 0);
-  time_t time_now = ASN1_UTCTIME_mktime(asn1_time);
-  ASN1_UTCTIME_free(asn1_time);
-  time_t time_after = ASN1_UTCTIME_mktime(X509_get_notAfter(ucert));
-  time_t time_diff = time_after - time_now ;
-  int length  = hours*60*60 + minutes*60;
-
-  if (time_diff < 0) {
-    Print(WARN) << std::endl << "ERROR: Your certificate expired "
-                << asctime(localtime(&time_after)) << std::endl;
-    
-    return true;
-  } 
-  
-  if (hours && time_diff < length) {
-    Print(WARN) << std::endl << "Warning: your certificate and proxy will expire "
-                << asctime(localtime(&time_after))
-                << "which is within the requested lifetime of the proxy"
-                << std::endl;
-    return false;
-  }
-  
-  if (!quiet) {
-    time_t time_after_proxy;
-    time_after_proxy = time_now + length;
-    
-    Print(INFO) << "Your proxy is valid until "
-                << asctime(localtime(&time_after_proxy)) << std::flush;
-  }
 
   return false;
 }
@@ -1098,6 +1106,39 @@ bool Client::checkstats(char *file, int mode)
   return true;
 }
 
+static bool check_validity_dates(X509 const* cert, int& time_left, std::string& error)
+{
+  assert(cert != nullptr);
+
+  time_left = 0;
+  error.clear();
+
+  ASN1_TIME const* not_before = X509_get0_notBefore(cert);
+  ASN1_TIME const* not_after = X509_get0_notAfter(cert);
+  int start_cmp = X509_cmp_current_time(not_before);
+  int end_cmp = X509_cmp_current_time(not_after);
+
+  if (start_cmp == 0 || end_cmp == 0) {
+    error = "Cannot check validity of certificate dates";
+    return false;
+  }
+
+  if (start_cmp > 0) {
+    error = "Certificate is not yet valid; validity starts on " + to_string(not_before);
+    return false;
+  }
+
+  int days{0}, secs{0};
+  ASN1_TIME_diff(&days, &secs, nullptr, not_after);
+  time_left = days * 24 * 60 * 60 + secs;
+
+  if (end_cmp < 0) {
+    error = "Certificate has expired on " + to_string(not_after);
+    return false;
+  }
+
+  return true;
+}
 
 bool Client::pcdInit() 
 {
@@ -1109,26 +1150,30 @@ bool Client::pcdInit()
   OpenSSL_add_all_ciphers();
   PKCS12_PBE_add();
   
-  if (!determine_filenames(&cacertfile, &certdir, &outfile, &certfile, &keyfile, noregen ? 1 : 0))
-    goto err;
+  if (!determine_filenames(&cacertfile, &certdir, &outfile, &certfile, &keyfile, noregen ? 1 : 0)) {
+    Error();
+    return false;
+  }
 
   if (!certfile){
-    Print(ERROR) << "ERROR: Coudln't find valid credentials to generate a proxy." << std::endl;
-    goto err;
+    Print(ERROR) << "ERROR: Couldn't find valid credentials to generate a proxy." << std::endl;
+    Error();
+    return false;
   }
 
 
-  if (certfile == keyfile) 
+  if (certfile == keyfile) {
     keyfile = strdup(certfile);
+  }
 
   if (!noregen) {
-    if (certfile)
+    if (certfile) {
       setenv("X509_USER_CERT", certfile, 1);
-
-    if (keyfile)
+    }
+    if (keyfile) {
       setenv("X509_USER_KEY", keyfile, 1);
-  }
-  else {
+    }
+  } else {
     if (outfile) {
       setenv("X509_USER_CERT", outfile, 1);
       setenv("X509_USER_KEY", outfile, 1);
@@ -1139,33 +1184,42 @@ bool Client::pcdInit()
 
   if (!checkstats(certfile, S_IXUSR | S_IWGRP | S_IXGRP | S_IWOTH | S_IXOTH) ||
       !checkstats(keyfile, S_IXUSR | S_IRGRP | S_IWGRP | S_IXGRP | S_IRGRP |
-                  S_IWOTH | S_IXOTH))
-    exit(1);
+                  S_IWOTH | S_IXOTH)) {
+    Error();
+    return false;
+  }
   
   Print(DEBUG) << "Files being used:" << std::endl 
                << " CA certificate file: " << (cacertfile ? cacertfile : "none") << std::endl
-               << " Trusted certificates directory : " << (certdir ? certdir : "none") << std::endl
-               << " Proxy certificate file : " << (outfile ? outfile : "none") << std::endl
+               << " Trusted certificates directory: " << (certdir ? certdir : "none") << std::endl
+               << " Proxy certificate file: " << (outfile ? outfile : "none") << std::endl
                << " User certificate file: " << (certfile ? certfile : "none") << std::endl
-               << " User key file: " << (keyfile ? keyfile : "none") << std::endl
-               << "Output to " << outfile << std::endl;
+               << " User key file: " << (keyfile ? keyfile : "none") << std::endl;
 
-  if (!load_credentials(certfile, keyfile, &ucert, &cert_chain, &private_key, pw_cb))
-    goto err;
+  if (!load_credentials(certfile, keyfile, &ucert, &cert_chain, &private_key, pw_cb)) {
+    Error();
+    return false;
+  }
 
   if (!quiet) {
-    char * s = NULL;
-    s = X509_NAME_oneline(X509_get_subject_name(ucert),NULL,0);
+    char* s = X509_NAME_oneline(X509_get_subject_name(ucert),NULL,0);
     Print(INFO) << "Your identity: " << s << std::endl;
     OPENSSL_free(s);
   }
 
-  status = true;
-  
- err:
-  Error();
-  return status;
-  
+  int time_left;
+  std::string error;
+  if (!check_validity_dates(ucert, time_left, error)) {
+    Print(ERROR) << "\nERROR: " << error << "\n\n";
+    Error();
+    return false;
+  }
+
+  if (time_left < hours * 60 * 60 + minutes * 60) {
+    Print(WARN) << "\nWARNING: proxy lifetime limited to issuing credential lifetime\n";
+  }
+
+  return true;
 }
 
 void Client::Error() 
