@@ -27,8 +27,6 @@
 #include <sys/types.h>
 #include <sys/utsname.h>
 #include <sys/stat.h>
-#include <sys/mman.h>
-#include <sys/wait.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <time.h>
@@ -40,45 +38,17 @@
 #include <ctype.h>
 #include <fcntl.h>
 #include <errno.h>
-#include <signal.h>
 #include <stdio.h>
+#include <signal.h>
 
 #include "log.h"
 #include "streamers.h"
-
-#define LOG_COMMAND       'L'
-#define SET_OPTION        'O'
-#define ACTIVATE_BUFFER   'A'
-#define DEACTIVATE_BUFFER 'D'
+#include "doio.h"
 
 static char *typenames[] = { "STARTUP", "REQUEST", "RESULT" };
 
 static char *levnames[] = { "LOG_ERROR", "LOG_WARN", "LOG_INFO", "LOG_DEBUG", "NONE"};
 
-static pid_t loggerprocess = 0;
-static pid_t ownerprocess = 0;
-
-static void killogger(void);
-
-extern void *FILEStreamerAdd(void *h);
-extern void *SYSLOGStreamerAdd(void *h);
-
-void SetOwner(pid_t pid)
-{
-  ownerprocess = pid;
-  atexit(killogger);
-}
-
-static void killogger(void)
-{
-  int status;
-
-  if (loggerprocess && (ownerprocess == getpid())) {
-    kill(loggerprocess, SIGKILL);
-    waitpid(loggerprocess, &status, 0);
-  }
-}
-
 struct OutputStream {
   void   *userdata;
   char   *name;
@@ -98,23 +68,15 @@ struct LogInfo {
   const char *format;
   const char *dateformat;
   const char *service;
-  int         fd;
   struct OutputStream *streamers;
 };
 
-static void Logger(void *data, int fd);
-static void Evaluate(struct LogInfo *li, char *buffer);
-static void Activate(struct LogInfo *li, char *name);
-static void Deactivate(struct LogInfo *li, char *name);
-static void SetOption(struct LogInfo *li, char *name, char *value);
-static void LogCommand(struct LogInfo *li, char *message);
-
 void *LogInit()
 {
   struct LogInfo *info = NULL;
 
   info = calloc(1, sizeof(struct LogInfo));
-  info->fd = -1;
+
   if (info) {
     FILEStreamerAdd(info);
     SYSLOGStreamerAdd(info);
@@ -123,217 +85,26 @@ void *LogInit()
   return info;
 }
 
-int LogBuffer(FILE *f, void *logh, loglevels lev, logtypes type, const char *format)
-{
-  int id;
-  struct stat st;
-  char *mem;
-
-  if (!f || !logh || !format) return 1;
-
-  id = fileno(f);
-
-  if (!fstat(id, &st)) {
-    if ((mem = (char *)mmap(0, st.st_size, PROT_READ, 0, id, 0))) {
-      LOGM(VARP, logh, type, lev, format, mem);
-      munmap(mem, st.st_size);
-      return 1;
-    }
-  }
-  return 0;
-}
-
-static int bwrite(int fd, const char * s) 
-{
-  int ret = -1;
-  
-  int slen = strlen(s);
-  int blen = sizeof(int) + slen;
-  
-  char * buffer = malloc(blen);
-  if(buffer) {
-    memcpy(buffer, &slen, sizeof(int));
-    memcpy(buffer + sizeof(int), s, strlen(s));
-    ret = write(fd, buffer, blen);
-  }
-  free(buffer);
-
-  return (ret != -1);
-}
-
-static int bread(int fd, char** buffer)
-{
-  int reload = 0;
-  int offset = 0;
-  int slen = 0;
-  ssize_t readbytes = -1;
-
-  do {
-    readbytes = read(fd, &slen, sizeof(int));
-  } while (readbytes < 0 && (errno == EINTR
-#ifdef ERESTART
-                               || errno == ERESTART
-#endif
-                               ));
-
-  if (readbytes != sizeof(int))
-    return 0;
-
-
-  *buffer = malloc(slen * sizeof(char) + 1);
-
-  if (*buffer) {
-    while (offset < slen) {
-      do {
-        readbytes = read(fd, *buffer + offset, (slen - offset > PIPE_BUF ? PIPE_BUF : slen - offset));
-      } while (readbytes < 0 && (errno == EINTR
-#ifdef ERESTART
-                                   || errno == ERESTART
-#endif
-                                   ));
-      offset += readbytes;      
-    }
-
-    (*buffer)[offset] = '\0';
-  }
-  return (reload ? -1 : offset);
-}
-
-void StartLogger(void *data, int code)
-{
-  struct LogInfo * li = (struct LogInfo *)data;
-
-  int in = -1, out = -1;
-
-#ifdef HAVE_MKFIFO
-  char fifo[30];
-
-  sprintf(fifo, "/tmp/voms_log_fifo_%i", code);
-  
-  if(mkfifo(fifo, S_IRUSR | S_IWUSR))
-  {
-    if(errno != EEXIST)
-    {
-      printf("Unable to make fifo : %s\n", strerror(errno));
-      exit(1);
-    }
-  }
-#else
-  int fd[2];
-
-  if (pipe(fd))
-  {
-    printf("Unable to open pipe : %s\n", strerror(errno));
-    exit(1);
-  }
-  in = fd[0];
-  out = fd[1];
-#endif
-
-  pid_t pid = fork();
-
-  if (pid) {
-    loggerprocess = pid;
-/*     ownerprocess = getpid(); */
-/*     atexit(killogger); */
-    if (out == -1)
-      out = open(fifo, O_WRONLY);
-    li->fd = out;
-    if (in != -1)
-      close(in);
-  }
-  else {
-    if (out != -1)
-      close(out);
-    if (in == -1)
-      in  = open(fifo, O_RDONLY);
-    Logger(data, in);
-  }
-}
-
-static void Logger(void *data, int fd) 
-{
-  char *buffer = NULL;
-  int result;
-
-  struct LogInfo *li = data;
-
-  if (!li)
-    return;
-
-  while(1) {
-    result = bread(fd, &buffer);
-
-    if (result) {
-      if (buffer) {
-        Evaluate(li, buffer);
-        free(buffer);
-        buffer = NULL;
-      }
-    }
-  }
-}
-
-static void Evaluate(struct LogInfo *li, char *buffer)
-{
-  char *name = NULL;
-  char *pos = NULL;
-
-  switch(*buffer) {
-  case ACTIVATE_BUFFER:
-    name = buffer+1;
-    Activate(li, name);
-    break;
-
-  case DEACTIVATE_BUFFER:
-    name = buffer+1;
-    Deactivate(li, name);
-    break;
-
-  case SET_OPTION:
-    name = buffer+1;
-    pos = strchr(name, '=');
-    if (pos) {
-      *pos++ = '\0';
-      SetOption(li, name, pos);
-    }
-    break;
-
-  case LOG_COMMAND:
-    name = buffer+1;
-    LogCommand(li, name);
-    break;
-
-  default:
-    LogCommand(li, "Unknown logging command: ");
-    LogCommand(li, buffer);
-    break;
-  }
-}
-
 void LogOption(void *data, const char *name, const char *value)
 {
   struct LogInfo *li=(struct LogInfo *)data;
-  char *buffer = NULL;
 
-  buffer = malloc(strlen(name)+strlen(value)+3);
-  buffer[0]=SET_OPTION;
-  buffer[1]='\0';
-
-  buffer = strcat(buffer, name);
-  buffer = strcat(buffer, "=");
-  buffer = strcat(buffer, value);
-  bwrite(li->fd, buffer);
-  free(buffer);
+  struct OutputStream *stream = li->streamers;
 
+  while (stream) {
+    stream->optioner(stream->userdata, name, value);
+    stream = stream->next;
+  }
 }
 
 void LogOptionInt(void *data, const char *name, int value)
 {
 #define INTSIZE (((sizeof(int)*CHAR_BIT)/3)+2)
   static char val[INTSIZE];
+
+  memset(val, 0, INTSIZE);
+  (void)snprintf(val, INTSIZE, "%d", value);
 #undef INTSIZE
-  sprintf(val, "%d\0", value);
 
   LogOption(data, name, val);
 }
@@ -341,35 +112,6 @@ void LogOptionInt(void *data, const char *name, int value)
 void LogActivate(void *data, const char *name)
 {
   struct LogInfo *li=(struct LogInfo *)data;
-
-  char *buffer = NULL;
-
-  buffer = malloc(strlen(name)+2);
-  buffer[0]=ACTIVATE_BUFFER;
-  buffer[1]='\0';
-
-  buffer = strcat(buffer, name);
-  bwrite(li->fd, buffer);
-  free(buffer);
-}
-
-void LogDeactivate(void *data, const char *name)
-{
-  struct LogInfo *li=(struct LogInfo *)data;
-
-  char *buffer = NULL;
-
-  buffer = malloc(strlen(name)+2);
-  buffer[0] = DEACTIVATE_BUFFER;
-  buffer[1] = '\0';
-
-  buffer = strcat(buffer, name);
-  bwrite(li->fd, buffer);
-  free(buffer);
-}
-
-static void Activate(struct LogInfo *li, char *name)
-{
   struct OutputStream *stream;
 
   if (!li)
@@ -387,8 +129,9 @@ static void Activate(struct LogInfo *li, char *name)
   }
 }
 
-static void Deactivate(struct LogInfo *li, char *name)
+void LogDeactivate(void *data, const char *name)
 {
+  struct LogInfo *li=(struct LogInfo *)data;
   struct OutputStream *stream;
 
   if (!li)
@@ -406,29 +149,6 @@ static void Deactivate(struct LogInfo *li, char *name)
   }
 }
 
-
-static void SetOption(struct LogInfo *li, char *name, char *value)
-{
-  struct OutputStream *stream = li->streamers;
-
-  while (stream) {
-    stream->optioner(stream->userdata, name, value);
-    stream = stream->next;
-  }
-}
-
-static void LogCommand(struct LogInfo *li, char *message)
-{
-  struct OutputStream *stream = li->streamers;
-
-  while (stream) {
-    if (stream->active)
-      stream->outputter(stream->userdata, message);
-    stream = stream->next;
-  }
-}
-
-
 void *LogAddStreamer(void *data, const char *name,
                      void * (*i)(),
                      int (*o)(void *, const char *s), 
@@ -543,21 +263,17 @@ const char *LogFormat(void *data, const char *format)
   return oldfmt;
 }
 
-static int LogOutput(void *data, loglevels lev, const char *str)
+static int LogOutput(void *data, const char *str)
 {
-
   struct LogInfo *li=(struct LogInfo *)data;
 
-  char *buffer = NULL;
-
-  buffer = malloc(strlen(str)+2);
-  buffer[0] = LOG_COMMAND;
-  buffer[1] = '\0';
-
-  buffer = strcat(buffer, str);
-  bwrite(li->fd, buffer);
-  free(buffer);
+  struct OutputStream *stream = li->streamers;
 
+  while (stream) {
+    if (stream->active)
+      stream->outputter(stream->userdata, str);
+    stream = stream->next;
+  }
   return 1;
 }
 
@@ -585,13 +301,20 @@ static char *StringAdd(char *dest, const char *src, signed int len)
   return tmp;
 }
 
+int LogLevelMin(void *data, loglevels lev)
+{
+  struct LogInfo *li=(struct LogInfo *)data;
+
+  if (li && ((li->currlev >= lev) && (li->currlev != LEV_NONE)))
+    return 1;
+
+  return 0;
+}
+
 int LogMessageF(const char *func, int line, const char *file, void *data, loglevels lev, logtypes type, const char *format, ...)
 {
   va_list v;
   char *str = NULL;
-  int len = 0;
-  int plen;
-  int res = 0;
   struct LogInfo *li=(struct LogInfo *)data;
 
 
@@ -603,17 +326,12 @@ int LogMessageF(const char *func, int line, const char *file, void *data, loglev
 
   if (li) {
     if ((li->currlev >= lev) || (li->currlev == LEV_DEBUG)) {
-      do {
-        len += 50;
-        str = realloc(str, len);
+      int res = 0;
+
+      va_start(v, format);
+      str = vsnprintf_wrap(format, v);
+      va_end(v);
 
-        if (str) {
-          va_start(v, format);
-          plen = vsnprintf(str, len, format, v);
-          va_end(v);
-        }
-      } while (str && (plen>=len));
-      
       if (str) {
         res = LogMessage(data, lev, type, str, func, line, file);
         free(str);
@@ -651,7 +369,6 @@ static signed int GetLen(const char **message)
 int LogMessage(void *data, loglevels lev, logtypes type, const char *message, const char *func, int line, const char *file)
 {
   struct LogInfo *li=(struct LogInfo *)data;
-  signed int len;
   char *msgcopy = NULL;
 
   if (!data)
@@ -672,6 +389,7 @@ int LogMessage(void *data, loglevels lev, logtypes type, const char *message, co
       int mode = 0;
       char *str = NULL;
       char *holder = NULL;
+      signed int len = 0;
 
       msgcopy = strdup(message);
       holder = msgcopy;
@@ -855,7 +573,7 @@ int LogMessage(void *data, loglevels lev, logtypes type, const char *message, co
         format++;
       }
       str = StringAdd(str, "\n", -1);
-      LogOutput(data, lev, str);
+      LogOutput(data, str);
       free(str);
     }
   }
