@@ -162,8 +162,6 @@ proxy_verify_desc *setup_initializers(char *cadir)
 
   pvd  = (proxy_verify_desc*)     malloc(sizeof(proxy_verify_desc));
   pvxd = (proxy_verify_ctx_desc *)malloc(sizeof(proxy_verify_ctx_desc));
-  pvd->cert_store = NULL;
-
 
   if (!pvd || !pvxd) {
     free(pvd);
@@ -298,6 +296,7 @@ GSISocketClient::Open()
     ssl = SSL_new(ctx);
     setup_SSL_proxy_handler(ssl, cacertdir);
     SSL_set_bio(ssl, conn, conn);
+    SSL_set_tlsext_host_name(ssl, host.c_str());
 
     conn = NULL;
 
